/*
** Copyright (C) 2005 Softwink,  Inc. <beave@softwink.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"		/* From autoconf */
#endif

#include "tserial.h"		/* Serial control */
#include "version.h"
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <curses.h>

#ifdef HAVE_LIBMYSQLCLIENT
#include <mysql/mysql.h>
#endif

#ifdef HAVE_LIBMYSQLCLIENT
static MYSQL    mysql;
#endif

#include <sys/types.h>
#include <limits.h>

#define MAXPATH		255

/**************************************************/
/* Funcitons for ncurses (from iwar-ncurses.c)    */
/**************************************************/

void mainscreen( void );
void nstatus( const char * );
void ntimer( int timer );
void ncount( int ncount);
void nintro( void );
void npause( void );
void ninfo(const char *, int );  /* 1 = ERROR , 2 = WARN */
int nplot(long long , int , int );
void nright(int, int);
char *nsimpleform();
char *nfilename();

/**************************************************/
/* Functions used by iwar-engine.c                */
/**************************************************/

void usage( void );
void closetty( int );
void rowcolcheck( void );
void sendmodem( const char *, const char * );
void plushangup( void );
void loginfo( int, const char *,  const char * );
int  getnum( int , int, int, const char *,  const char * );


int portfd;
FILE *outfd;		/* logfile/output file */
FILE *banner;           /* banners file (system identification) */
FILE *blacklst;         /* blacklisted numbers  file */
FILE *iwarcfg;          /* iwar configuration file */
FILE *saveloadstate;    /* load/save state file */


char sendstring[128]="";
char modemqueue[128]="";
char fileout[MAXPATH]="iwar.log";

char tmp[128];
char tmp2[128];

long long dialnum;
long long ss;
long long es;
long long num[9999];
int savestate=0;
int i; int b;
int j=0;
int row=9;
int col=2;

/* Blacklist */

struct blacklist_struct
        {
         long long blacklistnum;
        } blacklist[1000];

int blacklistcount;

/* MySQL authenication information */

#ifdef HAVE_LIBMYSQLCLIENT
char mysqlusername[32];
char mysqlpassword[32];
char mysqlhost[32];
char mysqldatabase[32];
#endif

/***************************************************************************/
/*                         General Help/Usage                              */
/***************************************************************************/

void usage(void)
{
printf("\nIwar [Intelligent wardialer] VERSION %s - By Da Beave (beave@softwink.com)\n\n",VERSION);
printf("usage: iwar [parameters] -r [dial range]\n\n");
printf("\t -h  :  Prints this screen\n");
printf("\t -s  :  Speed/Baud rate [default 1200]\n");
printf("\t -p  :  Parity (None/Even/Odd) [default 'N'one]\n");
printf("\t -d  :  Data bits [default 8]\n");
printf("\t -t  :  TTY to use (modem) [default /dev/ttyS0]\n");
printf("\t -c  :  Use software handshaking (XON/XOFF)\n\t        [default is hardware flow control]\n");
printf("\t -f  :  Output log file [Default: iwar.log]\n");
printf("\t -e  :  Pre-dial string/NPA to scan [optional]\n");
printf("\t -g  :  Post-dial string [optional]\n");
printf("\t -a  :  Tone Location (Toneloc W; method) [default: disabled]\n");
printf("\t -r  :  Range to scan (ie - 5551212-5551313)\n");
printf("\t -x  :  Sequential dialing [Default: Random]\n");
printf("\t -F  :  Full logging (BUSY, NO CARRIER, etc)\n");
printf("\t -b  :  Disable banners check (determine OS)\n");

#ifdef HAVE_LIBMYSQLCLIENT
printf("\t -m  :  Log to MySQL database\n");
#endif

printf("\t -l  :  Load 'saved state' file (previously dialed numbers)\n\n");


}; 

/***********************************************************************/
/* closetty() - This is for trapping signals and/or errors.            */
/***********************************************************************/

void closetty(int sig)
{

   switch(sig) {
      case 1: 
	    ninfo("Error reading TTY.  Aborting!", 1);
	    endwin();
	    exit(1);
      break;
      
     case -2:
	    ninfo("Write error! Closing serial and aborting!", 1);
	    endwin();
	    exit(1);
      break;

    /* Hrmph.   This isn't really used - We just keep trying over and over */

    case -3:
           fprintf(stderr, "[No Dialtone! Closing serial/output!]\n");
	   ninfo("No dial tone! Aborting.....", 1);
	   exit(1);
     break;

    case -4:
      
      break;

    default:
           m_dtrtoggle(portfd);   /* DTR Hangup */
	   sprintf(tmp, "Signal %d received! Aborting.....", sig);
	   dialnum=0;  /* No reason to show number on a signal */
	   loginfo(0, tmp, ""); ninfo(tmp, 1);
	   endwin(); exit(1);
      break;

m_restorestate(portfd);        /* Restore state from "savestate" */
close(portfd);  	       /* Close serial/output file */
exit(0);
}
}  

/***********************************************************************/
/* sendmodem() -  How to send data to the modem                        */
/***********************************************************************/

void sendmodem(const char *sendstring, const char *modemqueue)
{

/* modemqueue is used for things like volumn changes,  speaker off, etc */

if (strcmp(modemqueue, "")!=0) 
   {
   write(portfd, modemqueue, strlen(modemqueue));
   modemqueue="";	/* Command was sent, clear the queue */
   sleep(1);
   }

/* Now do whatever you where originally instructed to do .... */

write(portfd, sendstring, strlen(sendstring));
}

/**********************************************************************/
/* getnum() - Gets a sequential/random number for dialing.  Also      */
/* check against blacklist numbers and pre-dialed numbers (loaded     */
/* from a file                                                        */
/**********************************************************************/

int getnum(int dialtype, int mysqllog, int tonedetect, const char *predial, const char *postdial)
{

/************************************/
/* Sequential dialing               */
/************************************/

int flag=1, k;

  if (dialtype == 1)
     {

     if (ss >= es)
        {
	dialnum=0; loginfo(0, "Sequential Scan Complete!", "");
	ninfo("Sequential Scan Completed!",1);
	endwin(); exit(0);
	}

dialnum=ss;

/* Check blacklist, make sure number isn't listed */

while (flag != 0)
{
     for (k=0; k<blacklistcount; k++)
        {
     	if (blacklist[k].blacklistnum == dialnum)
       		{
		nstatus("Blacklisted Number - Skipping...");
		loginfo(mysqllog, "BLACKLISTED", "");
                attron( COLOR_PAIR(15) | A_REVERSE);
                nplot(dialnum, row, col); rowcolcheck();
                attroff(COLOR_PAIR(15) | A_REVERSE);
		sleep(1); ss++; dialnum=ss; flag=1;
		break;
		} else {
		flag=0; 
		}
        }
}

/* If this is loaded from a previous state file, make sure we haven't
   already dialed the number previously */

flag=1;
while(savestate == 1 && flag != 0)
{
     for (k=3; k<j; k++)
         {
         if (num[k] == dialnum)
            {
            attron( COLOR_PAIR(14) | A_NORMAL );
            nplot(dialnum, row, col); rowcolcheck();
            attroff(COLOR_PAIR(14) | A_NORMAL );
            ss++; dialnum=ss; flag=1;
	    break;
            } else { 
	    flag=0; 
	    }
       }
}
    
/* Build our dial string.  Are we doing tone location? */

if (tonedetect==1 )
{ 
	/* Cant do old school tone location with a post dial string, 
	   so we ignore it.  */

	sprintf(sendstring, "ATDT%s%lldw;\r", predial, ss); /* Tone */
     	} else {
     	sprintf(sendstring, "ATDT%s%lld%s\r", predial, ss, postdial); 
    	}
        num[j]=ss; j++; ss++;
     	ncount(es-ss+1);
     	return 0;
        }

/***********************************/
/* Random Dialing                  */
/***********************************/

while (j < es-ss )
 {
         
	 //dialnum = (int)((double)rand() / ((double)RAND_MAX + 1) * es);
	 dialnum = (long long)((double)rand() / ((double)RAND_MAX + 1) * es);
         
	 if (dialnum >= ss)   /* Usable number? */
            { for (i=0; i < j; i++) if (num[i] == dialnum) flag=1; 
            
	    /* Check against blacklist */

	    for (i=0; i<blacklistcount; i++)
	    {
	    if (blacklist[i].blacklistnum == dialnum)
	            {
	            nstatus("Blacklisted Number - Skipping...");
		    attron( COLOR_PAIR(15) | A_REVERSE);
		    nplot(dialnum, row, col); rowcolcheck();
		    attroff(COLOR_PAIR(15) | A_REVERSE);
		    loginfo(mysqllog, "BLACKLISTED", "");
		    sleep(1);
		    num[j] = dialnum; j++;  flag=1;
		    break;
	            }
	         }

/* Number is good so far,  make sure we havent used it from a load file */

while(savestate == 1 && flag != 0)
{
     for (k=3; k<j; k++)
     {
	     if (num[k] == dialnum)
               {
               attron( COLOR_PAIR(14) | A_NORMAL );
               nplot(dialnum, row, col); rowcolcheck();
               attroff(COLOR_PAIR(14) | A_NORMAL );
               ss++; dialnum=ss;
               flag=1;
               break;
               } else {
               flag=0; 
	       }
      }
}


if (flag==0)
	{  
        num[j] = dialnum; j++; flag=0;
        if (tonedetect == 1)              /* Tone locations */
           {
           sprintf(sendstring, "ATDT%s%lldw;\r", predial, dialnum);
           } else {
           sprintf(sendstring, "ATDT%s%lld%s\r", predial, dialnum, postdial);
           }

           ncount(es-ss-j+1);
	   return 0;
         }
      flag=0;
     }
  }   // End while

/* If it's random and there is nothing left to do,  we've completed our 
   mission */

dialnum=0; loginfo(0, "Random Scan Complete", "");
ninfo("Random Scan Completed!",1);
endwin(); exit(0);
}


/***********************************************************************/
/* rowcolcheck() -  Checks to make make sure we are not trying to      */
/* information outside of the screen                                   */
/***********************************************************************/

void rowcolcheck( void )
{
	int maxrow;
	int maxcol;
	char tmp[30];
	getmaxyx(stdscr,maxrow,maxcol);  /* Current screen attributes */
	row++;

	sprintf(tmp, "%lld", dialnum);

	if ( row > maxrow-2)
		{
		row=9; col = col + strlen(tmp) + 2;
                } 

	/* If we're outside our range,   wipe the screen.   There's 
	   probably a better way to do this */

	if (col + strlen(tmp) >= maxcol)
               {
	       nstatus("Screen full: Clearing...");
               col=2; attron(COLOR_PAIR(10));
               for (i=9; i < maxrow; i++)
                 {
		 move(i,0); for (b=0; b < maxcol; b++)
                   {
                   addch(' ');		/* wipe it away.... */
                   }
                 } 
	       attroff(COLOR_PAIR(10));
               }


}

/**********************************************************************/
/* loginfo() - Function takes care of logging information to a ASCII  */
/* flat file and a MySQL database.  As it stands,  we always log to   */
/* a ASCII flat file,  MySQL is optional                              */
/**********************************************************************/

void loginfo(int mysqllog, const char *response, const char *ident )
{


int   length;
char *notsafe;		/* Pre-processed string from user */
char *encoded;          /* escape encoded for MySQL       */
char *sql;		/* Full query with encoded data   */

time_t now;

/* Standard ASCII flat file */

if ((outfd = fopen(fileout, "a")) == NULL)
        {
	fprintf(stderr, "\nERROR: Can't open %s for output\n\n", fileout);
	usage(); exit(1);
	}

now=time(NULL); strftime(tmp2, 127, "[ %H:%M:%S ]", localtime(&now));

/* Data is formated based on the information we need to log */

if (dialnum == 0)
   {
   fprintf(outfd, "%s %s\n", tmp2, response);
   } else {
   if (strlen(ident) == 0)  
      {
      fprintf(outfd, "%s %lld  %s\n", tmp2, dialnum, response);
      } else {
      fprintf(outfd, "%s %lld  %s [ %s ]\n", tmp2, dialnum, response, ident);
      }
}

/* If MySQL logging is enabled..... */

#ifdef HAVE_LIBMYSQLCLIENT

if ( mysqllog == 1)
	{
            if(mysql_init(&mysql) == NULL) 
            {
	    fprintf(outfd, "%s ERROR: %s\n", tmp2, mysql_error(&mysql));
	    ninfo("Can't init. MySQL",1);
	    fclose(outfd); m_restorestate(portfd);  /* Restore port */
            close(portfd);  endwin(); exit(1);
	    }

	mysql_options(&mysql,MYSQL_READ_DEFAULT_GROUP,"iwar");

	/* Connect to DB */

	if (!mysql_real_connect(&mysql, mysqlhost, mysqlusername, mysqlpassword, mysqldatabase,0,NULL,0))
	   {
	   fprintf(outfd, "%s ERROR: %s\n", tmp2, mysql_error(&mysql));
	   ninfo("Can't connect to the MySQL server",1);
	   fclose(outfd); m_restorestate(portfd);       
	   close(portfd);  endwin(); exit(1);
	   }

	if (mysql_select_db(&mysql, "iwar")) 
	  {
	  fprintf(outfd, "%s ERROR: %s\n", tmp2,  mysql_error(&mysql));
	  ninfo("Can't select the database", 1);
	  fclose(outfd); m_restorestate(portfd);     
	  fclose(outfd); close(portfd);  
	  endwin(); exit(1);
	  }

	/* I should probably use mysql_real_escape_string() here.  
	   It's all flat ASCII,  so it really shouldn't make much of
	   a difference. */

       notsafe=response;
       length=strlen(notsafe);
       encoded = (char *) malloc(length*2+1);  /* reserve plenty 'o room */
       sql = (char *) malloc(length*2+100);
       mysql_escape_string(encoded,notsafe,length);
       sprintf(sql, "INSERT INTO iwar values( now(), '%lld', '%s', '%s' )", dialnum, encoded, ident);

	 if(mysql_real_query(&mysql, sql, strlen(sql)))
	 	{
		fprintf(outfd, "%s ERROR: %s\n", tmp2, mysql_error(&mysql));
		ninfo("MySQL insert error!", 1);
                fclose(outfd); m_restorestate(portfd);      
		fclose(outfd); close(portfd);  
		endwin(); exit(1);
		}


	}

#endif

fflush(outfd);		/* flush! so if you're tail -f the log file! */
fclose(outfd);         

}

/**********************************************************************/
/* plushangup() - This is incase the modem won't hang up via DTR drop */
/* If you have to use this function,  prehaps you should get a better */
/* modem or fix your crap.  Try and _NOT_ use this.                   */
/**********************************************************************/

void plushangup()
{
sprintf(sendstring, "+++"); nstatus("+++");
sendmodem(sendstring, modemqueue);
sleep(2); nstatus("ATH"); sprintf(sendstring, "ATH\r");
sendmodem(sendstring, modemqueue); m_flush(portfd);
}


/***************************************************************************/
/* Welcome,  Welcome,  to main().  Where the magic and fun takes place     */
/***************************************************************************/

int main(int argc,  char **argv)
{

	struct banner_cfg
        {
        char search_string[128];
        char os_type[50];
        } bannercfg[1000];

	char iwar_option[30];
	char iwar_value[60];

	int serialtimeout=60;
	int bannertimeout=30;
	int bannermaxcount=2048;
	int bannersendcrtimeout=10;
	int bannersendcr=3;
	int bannercount=0;
	int conredial=5;
	int redial=3;
	int plushang=0;
	int sendcr=0;

	int key;                     /* move tom main */
	int bitcount=0;

	int c;	                     /* c for getopt */
	char ch;

	int connect=0;
	int nocarrier=0;
	int busy=0;
	int voice=0;
	int tonedetect=0;

	int tonesilence=0;

	/* Setup some default port settings */

	char tty[20]		= "/dev/ttyS0";
	char baudrate[7]	= "1200";
	char parity[2]		= "N";
	char bits[2]		= "8";
	char predial[41]="";
	char postdial[41]="";

	char modeminit[200]="";

	int dialtype=0;		/*  1 == sequential */
	int buflen;
	int waitin=0;
	int b;

	int maxrow;
	int maxcol;

	int oldmaxrow;
	int oldmaxcol;

	time_t now;

	char bannerfile[MAXPATH]=BANNER_FILE_PATH;
	char iwarconf[MAXPATH]=CONFIG_FILE_PATH;
	char blacklistfile[MAXPATH]=BLACKLIST_FILE_PATH;
	char statefile[MAXPATH]="";
	char tmpscanbuf[128]="";
	char scanbuf[128]="";

        int logtype=0;
	int ring=0;
	int remotering=0;
	int bannercheck=1;
	int beepflag=0;
	int mysqllog=0;
	char bannerbuf[1024];
	char iwarbuf[1024];
	int connectflag=0;

	int hwhandshake=1;
	int swhandshake=0;

	char tmpscanrange[31];
	char startscan[15];   
	char endscan[15];   

	char buf[128]="";		
	char buf2[123]="";


	fd_set fds;
	struct timeval tv;


/**************************************************************************/
/* Get options from the iwar configuration file                           */
/**************************************************************************/
if ((iwarcfg = fopen(iwarconf, "r")) == NULL)
  {
  fprintf(stderr, "\nERROR: Cannot open configuration file (%s)\n\n",iwarconf);
  usage(); exit(1);
  }

    while (fgets(iwarbuf, 1024, iwarcfg) != NULL)
     {
     if (iwarbuf[0] == '#') continue;		         /* Comments */
     sprintf(iwar_option, "%s", strtok(iwarbuf, " "));
     sprintf(iwar_value, "%s", strtok(NULL, " "));

    if (strcmp(iwar_option, "port")==0)       { 
    strncpy(tty, iwar_value, strlen(iwar_value)-1); }

    if (strcmp(iwar_option, "speed")==0)      { 
    strncpy(baudrate, iwar_value, strlen(iwar_value)-1); }

    if (strcmp(iwar_option, "parity")==0)     { 
    strncpy(parity, iwar_value, strlen(iwar_value)-1); }
    
    if (strcmp(iwar_option, "databits")==0)   { 
    strncpy(bits, iwar_value, strlen(iwar_value)-1); }

    if (strcmp(iwar_option, "init")==0)       { 
    strncpy(modeminit, iwar_value, strlen(iwar_value)-1); 
    strcat(modeminit, "\r"); 
    }
    
    if (strcmp(iwar_option, "banner_file")==0) 
    strncpy(bannerfile , iwar_value, strlen(iwar_value)-1 );

    if (strcmp(iwar_option, "blacklistfile")==0)
    strncpy(blacklistfile, iwar_value, strlen(iwar_value)-1);

    if (strcmp(iwar_option, "serial_timeout")==0) {
    strcpy(tmp, iwar_value);
    serialtimeout=atoi(tmp); 
    }

    if (strcmp(iwar_option, "remote_ring")==0) {
    strcpy(tmp, iwar_value);
    remotering=atoi(tmp);
    }

    if (strcmp(iwar_option, "tone_detect")==0) tonedetect=1;
    
    if (strcmp(iwar_option, "banner_timeout")==0) {
    strcpy(tmp, iwar_value);
    bannertimeout=atoi(tmp);
    }

    if (strcmp(iwar_option, "banner_maxcount")==0) {
    strcpy(tmp, iwar_value); 
    bannermaxcount=atoi(tmp);
    }

    if (strcmp(iwar_option, "banner_send_cr")==0) {
    strcpy(tmp, iwar_value); 
    bannersendcrtimeout=atoi(tmp);
    }

    if (strcmp(iwar_option, "banner_cr")==0) {
    strcpy(tmp, iwar_value);
    bannersendcr=atoi(tmp);
    }

    if (strcmp(iwar_option, "redial")==0) {
    strcpy(tmp, iwar_value);
    redial=atoi(tmp);
    }

    if (strcmp(iwar_option, "connect_redial")==0) {
    strcpy(tmp, iwar_value); 
    conredial=atoi(tmp);
    }

    if (strcmp(iwar_option, "beep")==0) beepflag=1;
    if (strcmp(iwar_option, "plushangup")==0) plushang=1;

    if (strcmp(iwar_option, "mysql_username")==0)
    strncpy(mysqlusername, iwar_value, strlen(iwar_value)-1); 

    if (strcmp(iwar_option, "mysql_password")==0)
    strncpy(mysqlpassword, iwar_value, strlen(iwar_value)-1);

    if (strcmp(iwar_option, "mysql_host")==0)
    strncpy(mysqlhost, iwar_value, strlen(iwar_value)-1); 

    if (strcmp(iwar_option, "mysql_database")==0)
    strncpy(mysqldatabase, iwar_value, strlen(iwar_value)-1); 

  }


/**************************************************************************/
/* Get command line options now                                           */
/**************************************************************************/

        while ((c = getopt (argc, argv, "r:s:p:l:d:t:e:g:f:r:hacxFbm")) != EOF)
	  {
	  switch(c)
		{
	 	case 'h':
		        usage();
			exit(0);
			break;
		case 's':

			/*
			Verify this is a valid rate they are passing to
			us.
			*/

			if (!strcmp(optarg, "110") || !strcmp(optarg,"300") ||
			!strcmp(optarg, "1200") || !strcmp(optarg,"2400")    ||
			!strcmp(optarg, "4800") || !strcmp(optarg,"9600")    ||
			!strcmp(optarg, "19200")|| !strcmp(optarg,"38400")   ||
			!strcmp(optarg, "57600")|| !strcmp(optarg,"115200"))
			  {
			  strncpy(baudrate, optarg, 7);
			  break;
			  }
			fprintf(stderr, "\nERROR: Invalid port speed.\n\n");
			usage();
			exit(1);
			break;
		case 'p':
			if (!strcmp(optarg, "N") || !strcmp(optarg,"E") ||
			!strcmp(optarg,"O"))
			  {
			  strncpy(parity, optarg, 1);
			  break;
			  }
			fprintf(stderr, "\nERROR: Invalid Parity.\n\n");
			usage();
			exit(1);
			break;
			
			/*
			Databits - Valid are 5,6,7,8
			*/

		case 'd':
			if (!strcmp(optarg, "5") || !strcmp(optarg,"6") ||
			!strcmp(optarg,"7") || !strcmp(optarg,"8"))
			  {
			  strncpy(bits, optarg, 1);
			  break;
			  }

			 fprintf(stderr, "\nERROR: Invalid Databits\n\n");
			 usage();
			 exit(1);
			 break;
                case 't':
                         if (strlen(optarg) > 20)
                         {
                         fprintf(stderr, "\nERROR: TTY is to long\n\n");
			 usage();
                         exit(1);
                         }
                        strncpy(tty, optarg, 20);
                        break;
		case 'l':
		        if (strlen(optarg) > MAXPATH)
			{
			fprintf(stderr, "\nERROR: State filename to long!\n\n");
			usage();
			exit(1);
			}
			strncpy(statefile, optarg, MAXPATH);
			break;
                case 'f':   
                        if (strlen(optarg)>(MAXPATH-1))
                        {
                        fprintf(stderr, "\nERROR: Log file to long\n\n");
			usage();
                        exit(0);
                        }
                        strncpy(fileout, optarg, strlen(optarg));
			break;
                case 'c':                      
                        swhandshake=1;
                        hwhandshake=0;
                        break;
		case 'r':
			if (strlen(optarg) > 40)
			{
			fprintf(stderr, "\nERROR: Scan range to long!\n\n");
			usage();
			exit(1);
			}
			strncpy(tmpscanrange, optarg, 30);
			break;
		case 'e':
			if (strlen(optarg) > 40)
			{
			fprintf(stderr, "\nError: Pre-dial to long\n\n");
			}
			strncpy(predial, optarg, strlen(optarg));
			break;	
		case 'g':
		        if (strlen(optarg) > 40)
		        {
		        fprintf(stderr, "\nError: Post-dial to long\n\n");
		        }
		        strncpy(postdial, optarg, strlen(optarg));
		        break;
		case 'a':
			tonedetect=1;
			break;
		case 'x':
			dialtype=1;
			break;
		case 'F':
		        logtype=1;
			break;
		case 'b':
			bannercheck=0;
			break;
		case 'm': 
			#ifdef HAVE_LIBMYSQLCLIENT
		        mysqllog=1;
			break;
			#else
			fprintf(stderr, "WARNING: MySQL was not compiled into Iwar! This option is disabled\n");
			break;
			#endif
		default:
			usage();
			exit(0);
			break;
		}
	     }


if (strcmp(statefile, "") == 0) 
{

/* convert user input into something usable. */

sprintf(startscan, "%s", strtok(tmpscanrange, "-")); ss=atoll(startscan);
sprintf(endscan, "%s", strtok(NULL, "-")); es=atoll(endscan);

/* Make sure user input is valid */

if (es == 0  || ss == es)
	{
	fprintf(stderr, "\nERROR: Invalid dial range specified!\n\n");
	usage();
	exit(1);
	}

if ( ss > es ) 
	{
	fprintf(stderr, "\nERROR: Start is GREATER than the ending!\n\n");
	usage();
	exit(1);
	}

if ( es > RAND_MAX && dialtype == 0)
	{
	fprintf(stderr, "\nERROR: Largest random number is %d.  Try using the predial string.\n\n", RAND_MAX);
	usage();
	exit(1);
	}

if ( es-ss > 100 ) {
	fprintf(stderr, "\nWARNING : You're dialing a 100 numbers or more!\n\n"); }

}


/**********************************/
/* Load system banners into array */
/**********************************/

printf("============================================================\n");
if (bannercheck==1)
	{
	if ((banner = fopen(bannerfile, "r")) == NULL)
	   {
	   fprintf(stderr, "\nERROR: Cannot load %s!\n\n", bannerfile);
	   exit(1);
	   }


        bannercount=0;
	  while (fgets(bannerbuf,1024,banner) != NULL)
          {
            if (bannerbuf[0] == '#') continue;
	    if (bannerbuf[0] == 10 ) continue;

            sprintf(bannercfg[bannercount].search_string, "%s", strtok(bannerbuf, "|"));
	    sprintf(tmp2, "%s", strtok(NULL, "|"));
	    strncpy(bannercfg[bannercount].os_type, tmp2, strlen(tmp2)-1 );
            bannercount++;
          }

printf("Remote identification banners loaded : %d\n", bannercount);
fclose(banner);
} 

/**************************************/
/* Load blacklisted number into array */
/**************************************/

if ((blacklst = fopen(blacklistfile, "r"))==NULL)
	{
	fprintf(stderr, "\nERROR: Cannot load blacklist file (%s)!\n\n", blacklistfile);
	exit(1);
	}

	blacklistcount=0;
	while (fgets(buf2,1024,blacklst) != NULL)
	{
		if (buf2[0] == '#') continue;
		if (buf2[0] == 10 ) continue;
	
	blacklist[blacklistcount].blacklistnum=atoll(buf2);
	blacklistcount++;
	}

fclose(blacklst);
printf("Blacklisted phone numbers            : %d\n", blacklistcount);

if (strcmp(statefile, "") != 0)
        {
	printf("Loading previous state file          : ");

	if ((saveloadstate = fopen(statefile, "r")) == NULL) 
	   {
	   printf("Can't read %s!\n", statefile);
	   exit(1);
	   }

	j=0;
	while(fgets(buf2,1024,saveloadstate) != NULL)
	{	
		if (buf2[0] == '#') continue;
		if (buf2[0] == 10 ) continue;
		num[j] = atoll(buf2);
		j++;
        }

	if ( j < 2)
	{
		printf("File appears to be incorrect [%d]\n", j);
		exit(1);
	}
savestate=1;

dialtype=num[0];                /* dial type */
ss=num[1];                      /* from saved state file */
es=num[2];

/* In case they save state again */

sprintf(startscan, "%lld", ss);
sprintf(endscan, "%lld", es);  

if ( ss == 0 || es == 0) 
	{	
	printf("File loaded,  but data is incorrect.\n");
	exit(1);
	}

printf("%d numbers loaded\n", j-3);
j=j-3;                          /* When loading a file,  j need to be 
                                   sub'd by 3 or it'll count dialtype, 
                                   ss, and es */
fclose(saveloadstate);


}


printf("============================================================\n");
printf("Firing up iwar version %s! ......\n\n", VERSION);

if ( dialtype==0 )
        {
        strcpy(tmp, "Random"); } else { strcpy(tmp, "Seq."); }


sleep(2);

/* Dump to the logfile out scan information */

if ((outfd = fopen(fileout, "a")) == NULL)
        {
        fprintf(stderr, "\nERROR: Can't open %s for output\n\n", fileout);
        usage(); exit(1);
        }

/********************************************/
/* Dump dialing information to our log file */
/********************************************/

fprintf(outfd, "\n\n------------------------------------------------------------------------------\n");
fprintf(outfd, "= Iwar version %s - By Da Beave (beave@softwink.com)\n", VERSION);
fprintf(outfd, "= Port Settings : %s,%s,%s (%s)\n", baudrate, bits, parity, tty);
fprintf(outfd, "= HW Handshaking: %d  | Tone location : %d\n", hwhandshake, tonedetect);

fprintf(outfd, "= Start of scan: %lld | End of scan: %lld (Total Numbers: %lld)\n", ss,es,es-ss);

if (strcmp(predial, "")==0) 
	{ 
	fprintf(outfd, "= Pre dial: [None] | ");
	} else { 
	fprintf(outfd, "= Pre dial: %s | ", predial);
	}

if (strcmp(postdial, "")==0)
	{
	fprintf(outfd, "Post Dial: [None]\n");
	} else { 
	fprintf(outfd, "Post Dial: %s\n", postdial);
	}

fprintf(outfd, "= Dial Type: %s | Logfile: %s\n",tmp, fileout);

now=time(NULL); strftime(tmp2, 127, "%H:%M:%S %F", localtime(&now));

fprintf(outfd, "= Scan Start: %s\n", tmp2);
fprintf(outfd, "------------------------------------------------------------------------------\n");
fflush(outfd);
fclose(outfd);

/**************************************************************/
/* Yay!  Time to open our serial port and let the phone begin */
/**************************************************************/

portfd = open(tty, O_RDWR);
if  (portfd == -1)
                {
                fprintf(stderr, "ERROR: Can't open %s.\n", tty);
		exit(1);
                }

m_savestate(portfd); 	/* Save current state & setup the port */
m_setparms(portfd,baudrate,parity,bits,hwhandshake,swhandshake);
m_nohang(portfd);       /* Do we need to do this? */
m_hupcl(portfd, 1);
m_flush(portfd);        /* Flush any old data out before we start */

/* What to do in the event of a signal */

signal (SIGHUP, &closetty);
signal (SIGINT, &closetty);
signal (SIGQUIT, &closetty);
signal (SIGTERM, &closetty);
signal (SIGABRT, &closetty);

/* Draw main screen/window.... */

mainscreen();   /* Draw,  draw, draw */
getmaxyx(stdscr,maxrow,maxcol);
oldmaxrow=maxrow;  oldmaxcol=maxcol;

move(1,20); attron(COLOR_PAIR(1)); printw("%s,%s,%s (%s) [%s]", baudrate, bits, parity, tty, tmp);
move(2,20); printw("%lld - %lld (Total: %d)", ss, es, es-ss);

if (strcmp(predial, "")==0)
	{ 
	strcpy(tmp, "[None]"); 
	} else { 
	strcpy(tmp, predial);
	}

if (strcmp(postdial, "")==0)
        {
        strcpy(tmp2, "[None]");
        } else {
        strcpy(tmp2, postdial);
        }

move(3,20); printw("%s / %s ", tmp, tmp2);

if (logtype==1)
        {
        strcpy(tmp2, "[F]");
        } else {
        strcpy(tmp2, "[N]");
        }

move(4,20); printw("%s %s", fileout, tmp2);
for (i=1;i<6;i++) nright(i,0);
refresh(); 

/* seed random number generator */

srand( (unsigned int)time( NULL ) );


/* We dial the first number here.  The reason;   before we enter terminal  */
/* mode/loop,  we want some modem input/output happening.   Before we dial */
/* we wait a second before throwing data at it.  Probably not needed bit,  */
/* but it gives me a warm fuzzy....                                        */

nintro();   /* Hi,  I'm Beave and I wrote this progra .... */

if (strcmp(modeminit,"")!=0)
	{
	sendmodem(modeminit, modemqueue);
	nstatus("Initializing Modem....");
	ninfo("Initializing Modem....", 2);
	sleep(1);
	m_flush(portfd);   /* Avoid "OK" for tone detection */

	}

getnum(dialtype, mysqllog, tonedetect, predial, postdial);
nstatus(sendstring);		
sendmodem(sendstring,modemqueue);

while(1)
{
	tv.tv_sec=1;			/* 1 second */
	tv.tv_usec=0;

        FD_ZERO(&fds );
        FD_SET(portfd, &fds);
        
	if (select(portfd+1, &fds, NULL, NULL, &tv) > 0)
            {
            buflen = read(portfd, buf, 1);
            if (buflen == -1) closetty(-1);
            waitin=0;              /* Got data, reset out nodata counter */

                for (i=0; i<strlen(buf); i++)
                  {
                  sprintf(tmpscanbuf, "%c", buf[i]);
                      ch=buf[i];
                      if (iscntrl(ch))        /* No cntrl chars */
                      {
		      strcpy(tmpscanbuf, "");
		      strcpy(scanbuf,"");
                      } else { 
		      strcat(scanbuf, tmpscanbuf);
		      }

            /* if connected,  count the bytes */

                if (connectflag==1 && bannercheck == 1 ) bitcount++; 
		if (bitcount > bannermaxcount)
		      {
		      nstatus("Max banner data received");
		      loginfo(mysqllog, "CONNECT", "Not Identified (Max. banner data received)");  
		      m_dtrtoggle(portfd);
		      sleep(conredial);
		      m_flush(portfd);
		      if ( plushang == 1) plushangup();
		      strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
		      getnum(dialtype, mysqllog, tonedetect, predial,  postdial);
		      nstatus(sendstring);
		      sendmodem(sendstring, modemqueue);
		      bitcount=0; ring=0;
		      }

	 /* USR Courier send "NO DIAL TONE, other modems do "NO DIALTONE" */

		 if (strstr(scanbuf, "NO DIALTONE") || strstr(scanbuf, "NO DIAL TONE"))
                      {
/*                      closetty(-3); */   /* To close/abort or keep trying? */
			nstatus("No Dialtone!");
			ninfo("NO DIALTONE! Waiting to retry......",2);
   	                m_dtrtoggle(portfd);      

    	                /* we resend the same number we attempted */
		        sprintf(tmp, "Redialing: %lld", dialnum);
		        nstatus(tmp);
		        sleep(redial);
		        nstatus(sendstring);
		        sendmodem(sendstring, modemqueue);
			strcpy(scanbuf,""); strcpy(tmpscanbuf, "");
                      }

		 if (strcmp(scanbuf, "CONNECT")==0)
		     {
		     nstatus("CONNECTED!");
		     if ( beepflag == 1 ) beep();
                     attron( COLOR_PAIR(11) | A_BLINK );
                     nplot(dialnum, row, col); rowcolcheck();
		     attroff( COLOR_PAIR(11) | A_BLINK );
		     connect++; nright(1, connect);
		     if (bannercheck == 0)
		     {
		     loginfo(mysqllog, "CONNECTED", "");
		     m_dtrtoggle(portfd); /* DTR Hangup please */
		     sleep(conredial);
		     if ( plushang == 1) plushangup();
		     strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
		     getnum(dialtype, mysqllog, tonedetect, predial, postdial);
		     nstatus(sendstring);
		     sendmodem(sendstring, modemqueue);
		     } else { connectflag=1; }
		     }

                 if (strcmp(scanbuf, "NO CARRIER")==0 || strcmp(scanbuf, "NO ANSWER")==0 )
		     {
		     sendcr=0; ring=0;
		     m_flush(portfd);
		     
		     if (connectflag==1)   /* We'll get a NO CARRIER once
		                              a connection drops */
		       {  
		       connectflag=0; 
		       loginfo(mysqllog, "CONNECT", "Not Identified (disconnected)");
		       nstatus("Connection Dropped.");
		       sleep(2);
		       getnum(dialtype, mysqllog, tonedetect, predial, postdial);
		       nstatus(sendstring);
		       sendmodem(sendstring, modemqueue);
		       strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
		       } else {
		     nstatus("NO CARRIER");
		     attron( COLOR_PAIR(10) | A_NORMAL );
		     nplot(dialnum, row, col); rowcolcheck();
		     attroff( COLOR_PAIR(10) | A_NORMAL );
		     nocarrier++; nright(2, nocarrier);
		     if (logtype==1) loginfo(mysqllog, "NO CARRIER", "");
   	             sleep(redial);

                     getnum(dialtype, mysqllog, tonedetect, predial, postdial);
		     nstatus(sendstring);
                     sendmodem(sendstring, modemqueue);
		     strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
		     }

		     if (connectflag==2)
		        {
			strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
			connectflag=0;
			}
		     }

		 /* Count the number of RINGING it silence detections */
		 /* is enabled...                                     */

		 if (strcmp(scanbuf, "RINGING")==0 && remotering != 0 )
		     {
		     ring++;
		     sprintf(tmp, "RINGING (# %d)", ring);
		     nstatus(tmp);
		     } 

                 /* No silence detections..  Just report RINGING */

		 if (strcmp(scanbuf, "RINGING")==0 && remotering == 0)
		    {
		    nstatus("RINGING....");
		    }

                 /* If we are doing Toneloc style tone detection,  we look
		    for a "OK" after the ATDT5551212W;  */

		 if (strcmp(scanbuf, "OK")==0 && tonedetect==1 )
		    {
		    if ( beepflag == 1) beep();
		    loginfo(mysqllog, "TONE", "");
		    tonesilence++;
		    nright(5, tonesilence);
		    nstatus("Found Tone!");
	            attron( COLOR_PAIR(13) | A_STANDOUT );
                    nplot(dialnum, row, col); rowcolcheck();
                    attroff( COLOR_PAIR(13) | A_STANDOUT );
		    sleep(redial);
		    m_flush(portfd);
                    getnum(dialtype, mysqllog, tonedetect, predial, postdial);
                    m_dtrtoggle(portfd);
                    nstatus(sendstring);
                    sendmodem(sendstring, modemqueue);
                    strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
		    ring=0;
		    }

		 if (strcmp(scanbuf, "BUSY")==0)
		     {
		     nstatus("BUSY");
		     attron( COLOR_PAIR(12) | A_BOLD );
		     nplot(dialnum, row, col); rowcolcheck();
		     attroff( COLOR_PAIR(12) | A_BOLD );
		     busy++; nright(3, busy);
		     if (logtype==1) loginfo(mysqllog, "BUSY", "");
		     sleep(redial);		     
		     m_flush(portfd);       
		     getnum(dialtype, mysqllog, tonedetect, predial, postdial);
		     m_dtrtoggle(portfd);
		     nstatus(sendstring);
		     sendmodem(sendstring, modemqueue);
		     strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
		     ring=0;  /* yes,  weird */
		     }

		 if (strcmp(scanbuf, "VOICE")==0)
		     {
		     nstatus("VOICE");
		     attron( COLOR_PAIR(13) | A_UNDERLINE );
		     nplot(dialnum, row, col); rowcolcheck();
		     attroff( COLOR_PAIR(13)| A_UNDERLINE );
		     voice++; nright(4, voice);
		     if (logtype==1) loginfo(mysqllog, "VOICE", "");
		     sleep(redial);
	             getnum(dialtype, mysqllog, tonedetect, predial, postdial);
		     nstatus(sendstring);
		     sendmodem(sendstring, modemqueue);
		     strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
		     ring=0;
		     }

		 if (strcmp(scanbuf, "ERROR")==0)
		    {
		    nstatus("Modem ERROR");
		    ninfo("Modem ERROR", 1);
		    strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
		    }


  }
}

/* Add a byte count for connectflag==1 ,  that is,  if you dont get the
   banner within "banner_maxcount" (from configuration file),  
   then hang up.  */

if (connectflag==1)
	{
		for (b=0; b<bannercount; b++)
		{ 
		   if (strstr(scanbuf, bannercfg[b].search_string))
		      {
		      loginfo(mysqllog, "CONNECT", bannercfg[b].os_type);
		      nstatus("System Identified!");
		      if ( beepflag == 1 ) { beep(); beep(); }
		      m_dtrtoggle(portfd);
		      if ( plushang == 1) plushangup();
		      sleep(conredial);
		      m_flush(portfd);
		      strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
		      getnum(dialtype, mysqllog, tonedetect, predial, postdial);
		      nstatus(sendstring);
		      sendmodem(sendstring, modemqueue);
		      connectflag=0; bitcount=0;
	              }
                }

	if (waitin == bannersendcrtimeout && sendcr == 0)
	 {
	 nstatus("Sending CR's");
	 for (i=0; i<bannersendcr; i++) 
	  {
	  sprintf(tmp, "\r");
	  sendmodem(tmp, modemqueue); 
	  sleep(1);
	  bitcount=0;
	  sendcr=1;		/* Avoid re-sending CR's */
	  }
	 nstatus("Waiting for response...");
	 }
	 

	if (waitin == bannertimeout)
	 {
	 loginfo(mysqllog, "CONNECT", "Not Identified (stalled)");
	 nstatus("Data Trans. Stalled.");
         m_dtrtoggle(portfd);       // DTR hangup.
	 sleep(conredial);
	 if ( plushang == 1) plushangup();
	 strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
	 connectflag=0;
	 getnum(dialtype, mysqllog, tonedetect, predial, postdial);
	 nstatus(sendstring);
	 sendmodem(sendstring, modemqueue);
	 waitin=0;
	}
	}

/* Either we waited and got nothing,  or we hit the max number of rings */
/* This is for silence detections */

	ntimer(waitin);
        waitin++;

	if (waitin == serialtimeout && ring < remotering && ring != 0)
	{
	if ( beepflag == 1) beep();
	loginfo(mysqllog, "Possible Interesting Number", "");
	nstatus("Possible Interesting Number");
        tonesilence++;  
        nright(5, tonesilence);
        attron( COLOR_PAIR(13) | A_STANDOUT );
        nplot(dialnum, row, col); rowcolcheck();
        attroff( COLOR_PAIR(13) | A_STANDOUT );
	sendmodem("\r\r", modemqueue);
	sleep(conredial);
	m_dtrtoggle(portfd);
	m_flush(portfd);  /* get rid of the NO CARRIER */
	strcpy(scanbuf, ""); strcpy(tmpscanbuf, "");
	getnum(dialtype, mysqllog, tonedetect, predial, postdial);
	nstatus(sendstring);
	sendmodem(sendstring, modemqueue);
	waitin=0; ring=0;
	}

        if (waitin == serialtimeout || ring > remotering )
        {
	nstatus("Max. Rings Received");
	sendmodem("\r\r", modemqueue);   /* Hangs up/records a "NO CARRIER" */
	waitin=0; ring=0;
        sleep(conredial);
        getnum(dialtype, mysqllog, tonedetect, predial, postdial);
        nstatus(sendstring);
        sendmodem(sendstring, modemqueue);
	}

/* Semi-el'tard'o. This really, really needs to be redone.   The helps us
   redraw the screen if the users re-sizes it..  Poorly,  I might add... */

getmaxyx(stdscr,maxrow,maxcol);
if (oldmaxrow != maxrow || oldmaxcol != maxcol)
   {
   mainscreen();
   oldmaxrow=maxrow;  oldmaxcol=maxcol;

   if ( dialtype==0 )
           {
           strcpy(tmp, "Random"); } else { strcpy(tmp, "Seq."); }


   move(1,20); attron(COLOR_PAIR(1)); printw("%s,%s,%s (%s) [%s]",baudrate,bits,parity,tty,tmp);
   move(2,20); printw("%lld - %lld (Total: %d)", ss, es, es-ss);

   if (strcmp(predial, "")==0)
           {
           strcpy(tmp, "[None]");
           } else {
           strcpy(tmp, predial);
           }

   if (strcmp(postdial, "")==0)
              {       
              strcpy(tmp2, "[None]");
              } else {
              strcpy(tmp2, postdial);
	      }
    move(3,20); printw("%s / %s", tmp, tmp2);


          if (logtype==1)
            {
            strcpy(tmp2, "[F]");
            } else {
            strcpy(tmp2, "[N]");
            }

	  move(4,20); printw("%s %s", fileout, tmp2);
	  nright(1,connect);
	  nright(2,nocarrier);
	  nright(3,busy);
	  nright(4,voice);
	  nright(5,tonesilence);
          refresh();
}

/**************************************************************************/
/* Get keyboard input.  This allows us to do things like mark numbers,    */
/* toggle settings,  etc... etc..                                         */
/**************************************************************************/

key = getch();

	if (key != ERR)
		{
		
		if ( key == (int)'a' || key == (int)'A')
		   {
		   dialnum=0; loginfo(0, "User Abort", "");
		   nstatus("User Abort!");
		   ninfo("User Abort!",2);
		   endwin();
		   exit(0);
		   }

		if ( key == (int)'b' || key == (int)'B')
		   {
		   if ( beepflag == 0 )
		      { 
		      beepflag=1; 
		      ninfo("Beep Enabled", 2);
		      } else { 
		      beepflag=0;
		      ninfo("Beep Disabled",2);
		      }
		   }

		if ( key == (int)'0' )
		   {
		   ninfo("Modem Speaker Off",2);
		   strcpy(modemqueue, "ATM0\r\r");
		   }

		if ( key == (int)'1' )
		   { 
		   ninfo("Modem Speaker On [Volume 1]", 2);
		   strcpy(modemqueue, "ATM1L1\r\r");
		   }

		 if ( key == (int)'2' )
                   {
                   ninfo("Modem Speaker On [Volume 2]", 2);
                   strcpy(modemqueue, "ATM1L2\r\r");
                   }

   	        if ( key == (int)'3' )
                   {
                    ninfo("Modem Speaker On [Volume 3/Max.]", 2);
                    strcpy(modemqueue, "ATM1L3\r\r");
                    }

		if ( key == (int)'+' )
		   {
		   serialtimeout=serialtimeout+5;
		   sprintf(tmp, "Added 5 secs. [Now: %d]", serialtimeout);
		   nstatus(tmp);
		   }

		if ( key == (int)'-' )
		   {
		   serialtimeout=serialtimeout-5;
		   sprintf(tmp, "Subtract 5 secs. [Now: %d]", serialtimeout);
                   nstatus(tmp);
                   }


		if ( key == (int)'k' || key == (int)'K')
		   {
		    if (connectflag == 1)
                        {
                        m_dtrtoggle(portfd);   /* DTR Hangup */
                        if (plushang == 1 ) plushangup();
                        }
			sendmodem("\r\r", modemqueue);
			m_dtrtoggle(portfd);
                        m_flush(portfd);


		   strcpy(tmp, nsimpleform());
		   ninfo("Marked (Custom Input)", 2);
                   attron( COLOR_PAIR(11) | A_STANDOUT );
                   nplot(dialnum, row, col); rowcolcheck();
                   attroff( COLOR_PAIR(11) | A_STANDOUT );

		   loginfo(mysqllog, "MARK", tmp);
		   m_flush(portfd);  

		   getnum(dialtype, mysqllog, tonedetect, predial, postdial);
		   nstatus(sendstring);
		   sendmodem(sendstring, modemqueue);
		   }

		if ( key == (int)'s' || key == (int)'S')
		   {
		   strcpy(tmp, nfilename());
		   if ((saveloadstate = fopen(tmp, "w")) == NULL)
		           {
			   ninfo("Cant save state",1);
		           }

		   /* Note:  This need to become it's own function */

		   now=time(NULL); strftime(tmp2, 127, "[ %H:%M:%S ]", localtime(&now));
		   fprintf(saveloadstate, "# Iwar version %s.  Date: %s\n", VERSION, tmp2);
		   fprintf(saveloadstate, "# 1 - dialtype (1 Seq/0 Rand)\n");
		   fprintf(saveloadstate, "# 2 - startscan\n");
		   fprintf(saveloadstate, "# 3 - endscan \n");
		   fprintf(saveloadstate, "#     The rest are numbers dialed\n");
		   if (savestate == 0)   /* Never saved this before,  if  */
		   {                     /* I don't do this it'll rewrite */
                                         /* dialtype, startscan, etc      */
		   fprintf(saveloadstate, "%d\n", dialtype);
		   fprintf(saveloadstate, "%s\n", startscan);
		   fprintf(saveloadstate, "%s\n", endscan);
		   }

		   for (b=0; b<j; b++)
		   {
		   fprintf(saveloadstate, "%lld\n", num[b]);
		   }
		   fclose(saveloadstate);
		   sprintf(tmp2, "State saved to %s", tmp);
		   ninfo(tmp2, 2); strcpy(tmp, "");
		   }

		   if ( key == (int)'q' || key == (int)'Q')
		   {
		   strcpy(tmp, nfilename());
		   while ((saveloadstate = fopen(tmp, "w")) == NULL)
		   {
		   ninfo("Can't save state file, try again", 1);
		   strcpy(tmp, nfilename());
		   }
                   now=time(NULL); strftime(tmp2, 127, "[ %H:%M:%S ]", localtime(&now));
                   fprintf(saveloadstate, "# Iwar version %s.  Date: %s\n", VERSION, tmp2);

                   fprintf(saveloadstate, "# 1 - dialtype (1 Seq/0 Rand)\n");
	           fprintf(saveloadstate, "# 2 - startscan\n");
	           fprintf(saveloadstate, "# 3 - endscan \n");
	           fprintf(saveloadstate, "#     The rest are numbers dialed\n");
		   
		   if (savestate == 0)
		   {
	           fprintf(saveloadstate, "%d\n", dialtype);
	           fprintf(saveloadstate, "%s\n", startscan);
	           fprintf(saveloadstate, "%s\n", endscan);
		   }

	           for (b=0; b<j; b++)
	           {
	           fprintf(saveloadstate, "%lld\n", num[b]);
	           }
	           fclose(saveloadstate);
                   sprintf(tmp2, "State save to %s", tmp);
                   ninfo(tmp2, 2);
		   ninfo("Exiting Iwar!  Bye....",1);
		   endwin();
		   exit(0);
		   }

                   if ( key == (int)'p' || key == (int)'P') 
		   {
		   if (connectflag == 1)
		     {
		     m_dtrtoggle(portfd);  
                     if (plushang == 1 ) plushangup();
                     }
                   sendmodem("\r\r", modemqueue);
		   npause();
		   m_flush(portfd);
		   getnum(dialtype, mysqllog, tonedetect, predial, postdial);
		   nstatus(sendstring);
		   sendmodem(sendstring, modemqueue);
		   }

		if ( key == (int)' ' || key == (int)'c' || key == (int)'C' ||
		     key == (int)'f' || key == (int)'F' ||
		     key == (int)'t' || key == (int)'T' ||
		     key == (int)'v' || key == (int)'V' ||
		     key == (int)'x' || key == (int)'X' )
		   {

                   /* We _shouldnt_ be connected,  but just in case */

                   if (connectflag == 1)
                      {
                      m_dtrtoggle(portfd);   /* DTR Hangup */
                      if (plushang == 1 ) plushangup();
                      }
                      sendmodem("\r\r", modemqueue);  /* Hangup current dial */
		      m_dtrtoggle(portfd);   /* This clears out the */
                      m_flush(portfd);       /* NO CARRIER          */
                      waitin=0;

		   if ( key == (int)' ') { strcpy(tmp, "Marked (Quick)"); strcpy(tmp2, "Marked as interesting"); }
		   if ( key == (int)'c' || key == (int)'C' ) { strcpy(tmp, "Marked (CARRIER)"); strcpy(tmp2, "Marked - CARRIER"); }
		   if ( key == (int)'f' || key == (int)'F' ) { strcpy(tmp, "Marked (FAX)"); strcpy(tmp2, "Marked - FAX"); }
		   if ( key == (int)'t' || key == (int)'T' ) { strcpy(tmp, "Marked (TELCO/TONE)"); strcpy(tmp2, "Marked - TELCO/TONE"); }
		   if ( key == (int)'v' || key == (int)'V' ) { strcpy(tmp, "Marked (VOICE MAIL SYSTEM)"); strcpy(tmp2, "Marked - VOICE MAIL SYSTEM"); }
		   if ( key == (int)'x' || key == (int)'X' ) { strcpy(tmp, "Marked (PBX)"); strcpy(tmp2, "Marked - PBX"); }

		   loginfo(mysqllog, "MARK", tmp);
		   ninfo(tmp,2);
                   attron( COLOR_PAIR(11) | A_STANDOUT );
                   nplot(dialnum, row, col); rowcolcheck();
		   attroff( COLOR_PAIR(11) | A_STANDOUT );
		   sleep(redial);
	           getnum(dialtype, mysqllog, tonedetect, predial, postdial);
		   nstatus(sendstring);
		   
		   m_dtrtoggle(portfd);		/* Just to be sure the modem */
		   m_flush(portfd);             /* is cleared of status      */
		  
		  sendmodem(sendstring, modemqueue);
		   }

	     }
       }
}  /* End of main(),  sucker */
